/*
# GENERIC X-WINDOW-BASED TETRIS
#
#	die.c
#
###
#
#  Copyright (C) 1992	Qiang Alex Zhao
#			Computer Science Dept, University of Arizona
#			azhao@cs.arizona.edu
#
#			All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee is hereby granted,
#  provided that the above copyright notice appear in all copies and
#  that both that copyright notice and this permission notice appear in
#  supporting documentation, and that the name of the author not be
#  used in advertising or publicity pertaining to distribution of the
#  software without specific, written prior permission.
#
#  This program is distributed in the hope that it will be "playable",
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
*/

#include "tetris.h"

void
banner(field, str)
    field_t        *field;
    char           *str;
{
    int             mlen = strlen(str);
    int             x, y, w, ascent, descent;

    ascent = field->tfont->ascent;
    descent = field->tfont->descent;
    w = XTextWidth(field->tfont, str, mlen);
    x = (field->winwidth - w) / 2;
    y = (field->winheight - ascent - descent) / 2;

    XFillRectangle(disp, field->win, gc_w2, x - 5, y - ascent - 5,
	w + 10, ascent + descent + 10);
    XDrawString(disp, field->win, gc_ttx, x, y, str, mlen);
}

void
die(field)
    field_t        *field;
{
    time_t          tloc;
    score_t         myscore;
    struct passwd  *tmp;

    time(&tloc);

    banner(field, DIE_MESG);
    XFlush(disp);
    sleep(1);

    gethostname(myscore.myhost, NAMELEN);
    myscore.myhost[NAMELEN-1] = '\0';
    setpwent();
    tmp = getpwuid(getuid());
    if (tmp == NULL)		/* should not occur */
	sprintf(myscore.myname, "%d", getuid());
    else
	strncpy(myscore.myname, tmp->pw_name, NAMELEN);
    endpwent();
    myscore.myname[NAMELEN-1] = '\0';

    sprintf(myscore.score, "%9d", field->score);
    sprintf(myscore.level, "%3d", field->level);
    sprintf(myscore.lines, "%4d", field->lines);
    strcpy(myscore.mytime, asctime(localtime(&tloc)));
    fprintf(stderr, "\t%s", myscore.mytime);
    fprintf(stderr, "Your final score is %d, at level %d with %d raws.\n\n",
	   field->score, field->level, field->lines);

    addHighScore(&myscore, field->score);
    showHighScores(SHOWNSCORES);

    XCloseDisplay(disp);
    exit(0);
}

void 
addHighScore(myscore, snum)
    score_t        *myscore;
    int             snum;
{
    char            buff[2][SCORELEN];
    int             fd;
    int             tmp, ptmp, s1, s2;
    int             mycount = 0;
    Bool            saved = False, trickle = False;

    fd = open(SCOREFILE, O_CREAT | O_RDWR, 0644);
    if (fd == -1) {
	fprintf(stderr, "Cannot write the score-file!\n");
	return;
    }
    flock(fd, LOCK_EX);

    tmp = 0;
    ptmp = 1;
    bcopy((char *) myscore, buff[1], SCORELEN);

    while (read(fd, buff[tmp], SCORELEN) == SCORELEN) {
	sscanf(((score_t *)buff[tmp])->score, " %d", &s1);
	if (!saved && (s1 <= snum)) {
	    trickle = True;
	    saved = True;
	    mycount++;
	}
	if (!strncmp(myscore->myname,
		((score_t *)buff[tmp])->myname, NAMELEN)) {
	    mycount++;
	}
	/* Then check if we should trickle the score */
	if (trickle) {
	    lseek(fd, (off_t) -SCORELEN, L_INCR);
	    write(fd, buff[ptmp], SCORELEN);
	    ptmp = tmp;
	    tmp = (tmp + 1) % 2;
	}
	/*
	 * As we trickle, we add up records owned by me. Once we hit max,
	 * we throw it away, and stop trickling.
	 */
	if ((mycount > MAXSCORES) || ((mycount == MAXSCORES) && !trickle)) {
	    trickle = False;
	    break;
	}
    }	/* while */

    if (trickle) {
	write(fd, buff[ptmp], SCORELEN);
    }
    if (!saved && (mycount < MAXSCORES)) {
	write(fd, (char *) myscore, SCORELEN);
    }

    flock(fd, LOCK_UN);
    close(fd);
}

void 
showHighScores(ub)
    int             ub;
{
    int             fd, i;
    score_t         curs;

    fd = open(SCOREFILE, O_RDONLY, 0644);
    if (fd == -1)
	return;

    i = 0;
    fprintf(stderr, " # USER            SCORE   L    R  HOST         DATE\n");

    while (read(fd, (char *) &curs, SCORELEN) == SCORELEN) {
	i++;
	if (i<= ub)
	    fprintf(stderr, "%2d %-12s%9s %3s %4s  %-12s %-s",
		i, curs.myname, curs.score, curs.level, curs.lines,
		curs.myhost, curs.mytime);
    }
    close(fd);
    fprintf(stderr, "There are %d scores to date\n", i);
}

